@@ -203,9 +203,14 @@
   <do_scan_and_service>. Be more tolerant of some system errors. Fixed file
   descriptor leak in <action_copy>.
 
-    Last updated by Richard Gooch   18-AUG-2001: Dynamically load libnsl at
+    Updated by      Richard Gooch   18-AUG-2001: Dynamically load libnsl at
   run-time as needed, rather than linking. Based on patch from Adam J. Richter.
 
+    Last updated by Richard Gooch   19-AUG-2001: Switched to stat(2) in
+  <action_permissions> to avoid harmless false positives. Return 0 instead of
+  exiting if passwd or group entry not found in database. Do not exit in
+  <action_execute> if fork(2) fails.
+
 
 */
 #define _GNU_SOURCE
@@ -1077,7 +1082,7 @@ static void action_permissions (const struct devfsd_notify_struct *info,
     mode_t new_mode;
     struct stat statbuf;
 
-    if (lstat (info->devname, &statbuf) != 0)
+    if (stat (info->devname, &statbuf) != 0)
     {
 	SYSLOG (LOG_ERR, "error lstat(2)ing: \"%s\"\t%s\n",
 		info->devname, ERRSTRING);
@@ -1217,8 +1222,9 @@ static void action_execute (const struct devfsd_notify_struct *info,
 	break;
       case -1:
 	/*  Error  */
-	SYSLOG (LOG_ERR, "error forking\t%s\n", ERRSTRING);
-	exit (2);
+	SYSLOG (LOG_ERR, "error forking for programme: %s\t%s\n",
+		argv[0], ERRSTRING);
+	return;
 	/*break;*/
       default:
 	/*  Parent  */
@@ -1346,7 +1352,7 @@ static void action_copy (const struct devfsd_notify_struct *info,
 		symlink (source_link, destination);
 	    }
 	    return;
-	}
+	}   /*  Else not a symlink  */
 	if (source_stat.st_mode != dest_stat.st_mode)
 	    chmod (destination, source_stat.st_mode & ~S_IFMT);
 	if ( (source_stat.st_uid != dest_stat.st_uid) ||
@@ -1653,9 +1659,9 @@ static uid_t get_uid (const char *string)
 	return atoi (string);
     if ( ( pw_ent = getpwnam (string) ) == NULL )
     {
-	SYSLOG (LOG_ERR, "unknown user: \"%s\"\n", string);
-	SYSLOG (LOG_ERR, "exiting\n");
-	exit (1);
+	SYSLOG (LOG_ERR, "unknown user: \"%s\", defaulting to UID=0\n",
+		string);
+	return (0);
     }
     return (pw_ent->pw_uid);
 }   /*  End Function get_uid  */
@@ -1672,9 +1678,9 @@ static gid_t get_gid (const char *string)
 	return atoi (string);
     if ( ( grp_ent = getgrnam (string) ) == NULL )
     {
-	SYSLOG (LOG_ERR, "unknown group: \"%s\"\n", string);
-	SYSLOG (LOG_ERR, "exiting\n");
-	exit (1);
+	SYSLOG (LOG_ERR, "unknown group: \"%s\", defaulting to GID=0\n",
+		string);
+	return (0);
     }
     return (grp_ent->gr_gid);
 }   /*  End Function get_gid  */
@@ -1960,6 +1966,7 @@ static void expand_regexp (char *output, size_t outsize, const char *input,
     <numex> Number of subexpressions in the offset-array <<ex>>.
     <errfp> Diagnostic messages are written here. If this is NULL the global
     stderr is used instead.
+    [RETURNS] Nothing.
 */
 {
     const char last_exp = '0' - 1 + numex;
